/* Copyright (C) 2024 The Software Heritage developers
 * See the AUTHORS file at the top-level directory of this distribution
 * License: GNU Affero General Public License version 3, or any later version
 * See top-level LICENSE file for more information
 */

let originsFieldset, originsFields, addButton, originsCount, totalForms, individualFieldset, entityFieldset, submitterRadios;
const formRegex = RegExp(`form-(\\d+)-`, 'g');

/**
 * Add a new origin field.
 *
 * We clone the first origin field and append it to the origins fieldset with a
 * 'remove' button.
 * Then the number of origins (used to validate the django form set) is updated.
 */
function addOrigin(evt) {
  originsCount++;
  const newOrigin = originsFieldset.querySelector('.form-group:first-of-type').cloneNode(true);
  newOrigin.innerHTML = newOrigin.innerHTML.replace(formRegex, `form-${originsCount}-`);
  const inputNode = newOrigin.querySelector('input');
  inputNode.classList.remove('is-valid', 'is-invalid');
  inputNode.setAttribute('value', '');
  const inputGroup = buildInputGroup(inputNode);
  newOrigin.appendChild(inputGroup);
  originsFieldset.appendChild(newOrigin);
  totalForms.setAttribute('value', `${originsCount + 1}`);
}
/**
 * Remove an origin.
 *
 * Triggered by a button inside an added origin.
 */
function removeOrigin(evt) {
  evt.target.closest('.form-group').remove();
  originsFields = originsFieldset.querySelectorAll('.form-group');
  originsCount = 0;
  originsFields.forEach(
    (origin) => {
      const oldId = origin.querySelector('input').id;
      const newId = oldId.replace(formRegex, `form-${originsCount}-`);
      origin.querySelector('input').id = newId;
      origin.querySelector('label').setAttribute('for', newId);
      originsCount++;
    }
  );
  totalForms.setAttribute('value', `${originsCount}`);
}

/**
 * Append a remove button to all origin fields except the first one.
 */
function appendRemoveButton() {
  originsFieldset.querySelectorAll('.form-group:not(:first-of-type)').forEach(
    (origin) => {
      const originalInput = origin.querySelector('input');
      const inputGroup = buildInputGroup(originalInput.cloneNode(true));
      const errorMessage = origin.querySelector('.invalid-feedback');
      if (errorMessage) {
        inputGroup.appendChild(errorMessage);
      }
      originalInput.replaceWith(inputGroup);
    }
  );
}

/**
 * Build a new bootstrap input-group.
 *
 * @param  {HTMLElement} inputNode an origin HTML input tag
 * @return {HTMLElement} an origin HTML input tag with a delete button
 */
function buildInputGroup(inputNode) {
  const inputGroup = document.createElement('div');
  inputGroup.classList.add('input-group');

  const inputGroupAppend = document.createElement('div');
  inputGroupAppend.classList.add('input-group-append');

  const removeButton = document.createElement('button');
  removeButton.classList.add('btn', 'btn-sm', 'btn-outline-danger');
  removeButton.setAttribute('aria-label', '{% translate "Remove" %}');
  removeButton.setAttribute('type', 'button');
  removeButton.addEventListener('click', removeOrigin);

  const removeIcon = document.createElement('i');
  removeIcon.classList.add('mdi', 'mdi-close-circle');

  removeButton.appendChild(removeIcon);
  inputGroupAppend.appendChild(removeButton);
  inputGroup.appendChild(inputNode);
  inputGroup.appendChild(inputGroupAppend);
  return inputGroup;
}

/**
 * Switch between entity & individual fields.
 */
function toggleFields() {
  individualFieldset.classList.toggle('d-none');
  entityFieldset.classList.toggle('d-none');
}

document.addEventListener('DOMContentLoaded', (event) => {
  originsFieldset = document.querySelector('#origins-fieldset');
  originsFields = originsFieldset.querySelectorAll('.form-group');
  addButton = document.querySelector('#add-button');
  totalForms = document.querySelector('#id_form-TOTAL_FORMS');
  originsCount = originsFields.length - 1;
  addButton.addEventListener('click', addOrigin);
  // add a remove button to extra origins fields
  appendRemoveButton();

  individualFieldset = document.querySelector('#individual-fieldset');
  entityFieldset = document.querySelector('#entity-fieldset');
  submitterRadios = document.querySelectorAll('input[name="submitter_type"]');
  submitterRadios.forEach(
    (elt) => elt.addEventListener('change', toggleFields)
  );
  // handle cases where the browser checks another radio button than the default one
  if (document.querySelector('input[name="submitter_type"]:checked').value !== 'individual') {
    toggleFields();
  }
});
