# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import Any

from django.core.mail import send_mail
from django.template.loader import render_to_string
from django.utils.translation import gettext as _

from .forms import TakedownForm


def send_takedown_request_email(
    origins_data: list[dict[str, Any]], form_data: dict[str, Any]
) -> None:
    """Send a takedown request by mail to SWH.

    Args:
        origins_data: a list of origins
        form_data: request & contact details
    """
    send_mail(
        "Takedown request",
        render_to_string(
            "takedown_request.txt",
            {"origins_data": origins_data, "form_data": form_data},
        ),
        "swhweb@softwareheritage.org",  # TODO settings + env
        ["takedown@softwareheritage.org"],
    )


def send_takedown_acknowledgement_email(
    origins_data: list[dict[str, Any]], form_data: dict[str, Any]
) -> None:
    """Send a takedown request confirmation by mail to the submitter.

    Args:
        origins_data: a list of origins
        form_data: request & contact details
    """
    name = (
        form_data["individual_name"]
        if form_data["submitter_type"] == TakedownForm.INDIVIDUAL
        else form_data["entity_representative"]
    )
    send_mail(
        _("Confirmation of your takedown request"),
        render_to_string(
            "takedown_acknowledgement.txt",
            {"origins_data": origins_data, "name": name},
        ),
        "takedown@softwareheritage.org",  # TODO settings + env
        [form_data["contact_email"]],
    )
