# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information
from typing import Any

from django import forms
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.forms import BaseFormSet
from django.utils.translation import gettext as _

MAX_ORIGINS_NUM = 10


def validate_origin(origin: str) -> None:
    """Check if the origin exists in the archive.

    Args:
        origin: an origin

    Raises:
        ValidationError: the origin has not been found
    """
    from swh.web.utils.archive import lookup_origin

    try:
        lookup_origin(origin)
    except ObjectDoesNotExist:  # Should be a utils.exc.NotFoundExc
        raise ValidationError(_("This url is not archived by Software Heritage"))


class OriginForm(forms.Form):
    """Origin form."""

    origin = forms.CharField(
        label=_("Origin"),
        required=True,
        widget=forms.TextInput(attrs={"placeholder": _("URI")}),
        validators=[validate_origin],
    )


class BaseOriginFormSet(BaseFormSet):
    """Customized BaseFormSet for OriginForm."""

    def __init__(self, *args, **kwargs):
        """Customize error messages and reject empty origins."""
        super().__init__(*args, **kwargs)
        self.error_messages["too_many_forms"] = _(
            "A maximum of %(num)d origins can be submitted at a time."
        )
        self.error_messages["too_few_forms"] = _(
            "A minimum of %(num)d origin must be submitted."
        )
        for form in self.forms:
            form.empty_permitted = False


OriginFormSet = forms.formset_factory(
    OriginForm,
    formset=BaseOriginFormSet,
    max_num=MAX_ORIGINS_NUM,
    validate_max=True,
    min_num=1,
    validate_min=True,
    extra=0,
)


class TakedownForm(forms.Form):
    """Takedown request form."""

    INDIVIDUAL = "individual"
    ENTITY = "entity"
    SUBMITTER_TYPES = [(INDIVIDUAL, _("Individual")), (ENTITY, _("Legal entity"))]

    ASSOCIATION = "association"
    ENTITY_TYPES = [
        (ASSOCIATION, _("Association")),
    ]

    INDIVIDUAL_FIELDS = [
        "individual_name",
        "individual_activity",
        "individual_address",
        "individual_nationality",
        "individual_birthplace",
        "individual_birthdate",
    ]
    ENTITY_FIELDS = [
        "entity_type",
        "entity_name",
        "entity_representative",
        "entity_address",
    ]

    no_longer_accessible = forms.BooleanField(
        label=_(
            "I confirm that the origins mentioned above are no longer publicly available."
        ),
        required=True,
    )
    reasons = forms.CharField(
        label=_(
            "Reasons why the content should be taken down, "
            "including the supporting articles of law "
            "and full justification of the alleged facts."
        ),
        widget=forms.Textarea,
        required=True,
    )
    correspondence = forms.CharField(
        label=_(
            "Copy of the correspondence sent to the original author or editor "
            "of the allegedly infringing content, "
            "asking for its removal or modification; "
            "or justification of why they could not be contacted."
        ),
        widget=forms.Textarea,
        required=True,
    )
    contact_email = forms.EmailField(label=_("Contact email"), required=True)

    submitter_type = forms.ChoiceField(
        label=_("Type of submitter"),
        choices=SUBMITTER_TYPES,
        widget=forms.RadioSelect,
        required=True,
    )

    entity_type = forms.ChoiceField(
        label=_("Nature of the entity"),
        choices=ENTITY_TYPES,
        required=False,
        widget=forms.Select(attrs={"class": "custom-select"}),
    )
    entity_name = forms.CharField(label=_("Name"), required=False)
    entity_representative = forms.CharField(
        label=_("Legal representative"), required=False
    )
    entity_address = forms.CharField(
        label=_("Address of headquarters"),
        widget=forms.Textarea(attrs={"rows": 4}),
        required=False,
    )

    individual_name = forms.CharField(label=_("Full name"), required=False)
    individual_activity = forms.CharField(label=_("Activity"), required=False)
    individual_address = forms.CharField(
        label=_("Address"), widget=forms.Textarea(attrs={"rows": 4}), required=False
    )
    individual_nationality = forms.CharField(label=_("Nationality"), required=False)
    individual_birthplace = forms.CharField(label=_("Birthplace"), required=False)
    individual_birthdate = forms.DateField(
        label=_("Birthdate"),
        widget=forms.DateInput(attrs={"type": "date"}),
        required=False,
    )

    def __init__(self, *args, **kwargs):
        """Init and set defaults."""
        super().__init__(*args, **kwargs)
        self.initial["submitter_type"] = self.INDIVIDUAL

    def clean(self) -> None | dict[str, Any]:
        """Validate fields depending on `submitter_type`.

        ``individual_*`` fields are mandatory for individuals and the ``entity_*`` ones
        for entities. Useless fields are removed from the cleaned data.

        Returns:
            Cleaned form data.
        """
        cleaned_data = super().clean()
        if not cleaned_data:
            return cleaned_data

        submitter_type = cleaned_data.get("submitter_type")

        if submitter_type == self.INDIVIDUAL:
            required_fields = self.INDIVIDUAL_FIELDS
            useless_fields = self.ENTITY_FIELDS
        else:
            required_fields = self.ENTITY_FIELDS
            useless_fields = self.INDIVIDUAL_FIELDS

        for missing_field in [
            field for field in required_fields if not cleaned_data.get(field)
        ]:
            self.add_error(missing_field, ValidationError(_("This field is required")))
        [cleaned_data.pop(key, None) for key in useless_fields]
        return cleaned_data
