# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information
import pytest

from swh.web.takedowns.emails import (
    send_takedown_acknowledgement_email,
    send_takedown_request_email,
)


def test_takedown_request_individual(mailoutbox):
    data = {
        "submitter_type": "individual",
        "individual_name": "Individual Name",
        "individual_activity": "Individual Activity",
        "individual_address": "Individual Address",
        "individual_nationality": "French",
        "individual_birthplace": "Town",
        "individual_birthdate": "2000-01-01",
    }
    send_takedown_request_email({}, data)
    assert len(mailoutbox) == 1
    message = mailoutbox[0]
    assert "Individual Name" in message.body
    assert "Individual Activity" in message.body
    assert "Individual Address" in message.body
    assert "French" in message.body
    assert "Town" in message.body
    assert "2000-01-01" in message.body
    assert "Entity type" not in message.body


def test_takedown_request_entity(mailoutbox):
    data = {
        "submitter_type": "entity",
        "entity_name": "Entity Name",
        "entity_type": "Entity type",
        "entity_representative": "Firstname Representative",
        "entity_address": "Entity Address",
    }
    send_takedown_request_email({}, data)
    assert len(mailoutbox) == 1
    message = mailoutbox[0]
    assert "Entity Name" in message.body
    assert "Entity type" in message.body
    assert "Firstname Representative" in message.body
    assert "Entity Address" in message.body
    assert "Individual Name" not in message.body


@pytest.mark.parametrize(
    "submitter_type",
    ["individual", "entity"],
)
def test_takedown_request_common(submitter_type, mailoutbox):
    origins = [{"origin": "http://example.org"}, {"origin": "http://example.com"}]
    form_data = {
        "contact_email": "individual.name@example.com",
        "reasons": "My reasons",
        "correspondence": "My correspondence",
    }
    send_takedown_request_email(origins, form_data)
    assert len(mailoutbox) == 1
    message = mailoutbox[0]
    assert message.subject == "Takedown request"
    assert message.from_email == "swhweb@softwareheritage.org"
    assert message.to == ["takedown@softwareheritage.org"]
    assert "individual.name@example.com" in message.body
    assert "* http://example.org" in message.body
    assert "* http://example.com" in message.body
    assert "My reasons" in message.body
    assert "My correspondence" in message.body


def test_takedown_confirmation_individual(mailoutbox):
    data = {
        "contact_email": "individual.name@example.com",
        "submitter_type": "individual",
        "individual_name": "Individual Name",
    }
    send_takedown_acknowledgement_email({}, data)
    assert len(mailoutbox) == 1
    message = mailoutbox[0]
    assert "Dear Individual Name," in message.body


def test_takedown_confirmation_entity(mailoutbox):
    data = {
        "contact_email": "individual.name@example.com",
        "submitter_type": "entity",
        "entity_representative": "Firstname Representative",
    }
    send_takedown_acknowledgement_email({}, data)
    assert len(mailoutbox) == 1
    message = mailoutbox[0]
    assert "Dear Firstname Representative," in message.body


@pytest.mark.parametrize(
    "submitter_type",
    ["individual", "entity"],
)
def test_takedown_confirmation_common(submitter_type, mailoutbox):
    origins = [{"origin": "http://example.org"}, {"origin": "http://example.com"}]
    form_data = {
        "contact_email": "name@example.com",
        "reasons": "My reasons",
        "individual_name": "Individual Name",
        "entity_representative": "Firstname Representative",
        "submitter_type": submitter_type,
    }
    send_takedown_acknowledgement_email(origins, form_data)
    assert len(mailoutbox) == 1
    message = mailoutbox[0]
    assert message.subject == "Confirmation of your takedown request"
    assert message.from_email == "takedown@softwareheritage.org"
    assert message.to == ["name@example.com"]
    assert "* http://example.org" in message.body
    assert "* http://example.com" in message.body
