# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information
import pytest

from swh.web.takedowns.forms import OriginFormSet, TakedownForm


def test_origin_minimum():
    data = {
        "form-TOTAL_FORMS": "0",
        "form-INITIAL_FORMS": "0",
    }
    form_set = OriginFormSet(data)
    assert not form_set.is_valid()
    assert "A minimum of 1" in form_set.non_form_errors()[0]


def test_origin_maximum():
    data = {
        "form-TOTAL_FORMS": "11",
        "form-INITIAL_FORMS": "0",
    }
    form_set = OriginFormSet(data)
    assert not form_set.is_valid()
    assert "A maximum of 10" in form_set.non_form_errors()[0]


def test_origin_empty():
    data = {
        "form-TOTAL_FORMS": "2",
        "form-INITIAL_FORMS": "0",
        "form-0-origin": "http://example.org",
        "form-1-origin": "",
    }
    form_set = OriginFormSet(data)
    assert not form_set.is_valid()
    assert "required" in form_set.errors[1]["origin"][0]


def test_origin_invalid():
    data = {
        "form-TOTAL_FORMS": "2",
        "form-INITIAL_FORMS": "0",
        "form-0-origin": "http://example.org",
        "form-1-origin": "http://example.com",
    }
    form_set = OriginFormSet(data)
    assert not form_set.is_valid()
    assert "not archived" in form_set.errors[1]["origin"][0]


def test_origin_ok():
    data = {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "0",
        "form-0-origin": "http://example.org",
    }
    form_set = OriginFormSet(data)
    assert form_set.is_valid()


@pytest.fixture
def takedown_data():
    return {
        "no_longer_accessible": True,
        "reasons": "reasons...",
        "correspondence": "correspondence...",
        "contact_email": "email@example.org",
        "submitter_type": TakedownForm.INDIVIDUAL,
        "individual_name": "Individual Name",
        "individual_activity": "Individual Activity",
        "individual_address": "Individual Address",
        "individual_nationality": "Individual Nationality",
        "individual_birthplace": "Individual Birthplace",
        "individual_birthdate": "2000-01-01",
        "entity_type": TakedownForm.ASSOCIATION,
        "entity_name": "Entity name",
        "entity_representative": "Entity Representative",
        "entity_address": "Entity Address",
    }


@pytest.mark.parametrize(
    "missing_field",
    ["individual_name", "individual_activity", "individual_address"],
)
def test_takedown_clean_individual(takedown_data, missing_field):
    takedown_data["submitter_type"] = TakedownForm.INDIVIDUAL
    takedown_data.pop(missing_field)
    form = TakedownForm(takedown_data)
    assert not form.is_valid()
    assert missing_field in form.errors


@pytest.mark.parametrize(
    "missing_field",
    ["entity_type", "entity_name", "entity_representative", "entity_address"],
)
def test_takedown_clean_entity(takedown_data, missing_field):
    takedown_data["submitter_type"] = TakedownForm.ENTITY
    takedown_data.pop(missing_field)
    form = TakedownForm(takedown_data)
    assert not form.is_valid()
    assert missing_field in form.errors


@pytest.mark.parametrize(
    "submitter_type,removed_fields_prefix",
    [
        (TakedownForm.INDIVIDUAL, "entity_"),
        (TakedownForm.ENTITY, "individual_"),
    ],
)
def test_takedown_remove_other_submitter_type_fields(
    takedown_data, submitter_type, removed_fields_prefix
):
    takedown_data["submitter_type"] = submitter_type
    form = TakedownForm(takedown_data)
    assert form.is_valid()
    assert not [
        key
        for key, __ in form.cleaned_data.items()
        if key.startswith(removed_fields_prefix)
    ]
