# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information
from http import HTTPStatus

import pytest
from pytest_django.asserts import assertContains

from swh.web.tests.helpers import check_html_get_response, check_html_post_response


@pytest.fixture
def takedown_data_individual():
    return {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "0",
        "form-0-origin": "http://example.org",
        "no_longer_accessible": True,
        "reasons": "reasons...",
        "correspondence": "correspondence...",
        "contact_email": "email@example.org",
        "submitter_type": "individual",
        "individual_name": "Individual Name",
        "individual_activity": "Individual Activity",
        "individual_address": "Individual Address",
        "individual_nationality": "French",
        "individual_birthplace": "France",
        "individual_birthdate": "12/06/2024",
    }


@pytest.fixture
def takedown_data_entity():
    return {
        "form-TOTAL_FORMS": "1",
        "form-INITIAL_FORMS": "0",
        "form-0-origin": "http://example.org",
        "no_longer_accessible": True,
        "reasons": "reasons...",
        "correspondence": "correspondence...",
        "contact_email": "email@example.org",
        "submitter_type": "entity",
        "entity_name": "Entity Name",
        "entity_type": "association",
        "entity_representative": "Entity Representative",
        "entity_address": "Entity Address",
    }


def test_get_takedown_form(client):
    check_html_get_response(
        client, "/requests/takedown/", HTTPStatus.OK, template_used="takedown_form.html"
    )


def test_post_takedown_form_individual_ok(client, takedown_data_individual, mailoutbox):
    response = check_html_post_response(
        client,
        "/requests/takedown/",
        HTTPStatus.OK,
        data=takedown_data_individual,
        follow=True,
    )
    assertContains(response, "Your takedown request has been sent")
    assert len(mailoutbox) == 2
    assert mailoutbox[0].subject == "Takedown request"
    assert mailoutbox[1].subject == "Confirmation of your takedown request"


def test_post_takedown_form_entity_ok(client, takedown_data_entity, mailoutbox):
    response = check_html_post_response(
        client,
        "/requests/takedown/",
        HTTPStatus.OK,
        data=takedown_data_entity,
        follow=True,
    )
    assertContains(response, "Your takedown request has been sent")
    assert len(mailoutbox) == 2
    assert mailoutbox[0].subject == "Takedown request"
    assert mailoutbox[1].subject == "Confirmation of your takedown request"


@pytest.mark.parametrize(
    "empty_field",
    [
        "individual_name",
        "individual_activity",
        "individual_address",
        "individual_birthplace",
        "individual_birthdate",
        "individual_nationality",
    ],
)
def test_post_takedown_form_individual_ko(
    client, takedown_data_individual, empty_field
):
    changes = {empty_field: ""}
    takedown_data_individual.update(changes)
    response = check_html_post_response(
        client,
        "/requests/takedown/",
        HTTPStatus.OK,
        data=takedown_data_individual,
        follow=True,
    )
    assertContains(response, "Please fix the errors in the form")


@pytest.mark.parametrize(
    "empty_field",
    ["entity_name", "entity_representative", "entity_type", "entity_address"],
)
def test_post_takedown_form_entity_ko(client, takedown_data_entity, empty_field):
    changes = {empty_field: "", "submitter_type": "entity"}
    takedown_data_entity.update(changes)
    response = check_html_post_response(
        client,
        "/requests/takedown/",
        HTTPStatus.OK,
        data=takedown_data_entity,
        follow=True,
    )
    assertContains(response, "Please fix the errors in the form")
