# Copyright (C) 2024  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

from typing import TYPE_CHECKING

from django.contrib import messages
from django.shortcuts import redirect, render
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views.decorators.http import require_http_methods

from .emails import send_takedown_acknowledgement_email, send_takedown_request_email
from .forms import OriginFormSet, TakedownForm

if TYPE_CHECKING:
    from django.http import HttpRequest, HttpResponse


@require_http_methods(["GET", "POST"])
def takedown_form(request: "HttpRequest") -> "HttpResponse":
    """Takedown request.

    Displays the takedown request form and handle its submission. It the form is valid:
    * its content is sent to our "takedown" email address
    * a confirmation email is sent to the submitter

    Args:
        request: an ``HttpRequest``

    Returns:
        The takedown form.
    """
    if request.method == "POST":
        form = TakedownForm(request.POST)
        origins = OriginFormSet(request.POST)
        if origins.is_valid() and form.is_valid():
            send_takedown_request_email(origins.cleaned_data, form.cleaned_data)
            send_takedown_acknowledgement_email(origins.cleaned_data, form.cleaned_data)
            messages.success(
                request,
                _(
                    "Your takedown request has been sent, "
                    "we will get back to you as soon as possible."
                ),
            )
            return redirect(reverse("takedown-form"))
    else:
        origins = OriginFormSet()
        form = TakedownForm()

    return render(
        request,
        "takedown_form.html",
        {"form": form, "origins": origins},
    )
